package main

import (
	"flag"
	"fmt"

	"os"
	"os/signal"
	"regexp"
	"strings"
	"syscall"

	"github.com/valyala/fasthttp"
)

type _Route struct {
	r      *regexp.Regexp
	m      string
	h      func(*fasthttp.RequestCtx, string)
	regstr string
}

type RouteStore []_Route

func (me *RouteStore) AddRoute(regstr string, method string, h func(*fasthttp.RequestCtx, string)) {
	r, _ := regexp.Compile(regstr)
	*me = append(*me, _Route{r: r, m: method, h: h, regstr: regstr})
}

type CallbackApiSvrCommand struct {
	set *flag.FlagSet

	apiReadBufferSize  int
	apiWriteBufferSize int
	rs                 RouteStore

	ListenAddr   *string
	channelkey   *string
	masterSecret *string
}

func NewCallbackApiSvrCommand() *CallbackApiSvrCommand {
	cmd := &CallbackApiSvrCommand{}
	cmd.set = flag.NewFlagSet(os.Args[0], flag.PanicOnError)

	cmd.ListenAddr = cmd.set.String("listen_addr", "127.0.0.1:80", "鉴听地址端口")
	cmd.channelkey = cmd.set.String("key", "127.0.0.1:80", "极光渠道channelkey")
	cmd.masterSecret = cmd.set.String("secret", "127.0.0.1:80", "极光渠道masterSecret")

	cmd.apiReadBufferSize = 1000 * 10 //  "http读缓冲大小", 默认值调小，避免API模块OOM
	cmd.apiWriteBufferSize = 1000 * 2 // "http写缓冲大小", 默认值调小，避免API模块OOM

	return cmd
}

func (this *CallbackApiSvrCommand) Run(args []string) {
	this.set.Parse(args)
	fmt.Println("CallbackApiSvrCommand Run")
	fmt.Println("listen_addr:", *this.ListenAddr)
	fmt.Println("channelkey:", *this.channelkey)
	fmt.Println("masterSecret:", *this.masterSecret)

	// 测试接口 /my/callback/api
	this.rs.AddRoute("(/my/callback/api)", "GET", this.Check)

	apiServer := &fasthttp.Server{
		Handler:         this.Serve,
		ReadBufferSize:  this.apiReadBufferSize,
		WriteBufferSize: this.apiWriteBufferSize,
	}

	if err := apiServer.ListenAndServe(*this.ListenAddr); err != nil {
		fmt.Println("CallbackApiSvrCommand server ListenAndServe err:", err)
		os.Exit(1)
	}
	ch := make(chan os.Signal)
	signal.Notify(ch, syscall.SIGINT, syscall.SIGTERM, syscall.SIGKILL)
	<-ch

}

func (this *CallbackApiSvrCommand) Serve(ctx *fasthttp.RequestCtx) {
	method := strings.ToUpper(string(ctx.Method()))
	uri := string(ctx.RequestURI())
	ctx.SetContentType("text/plain; charset=utf8")
	fmt.Println("come", method, uri)
	for _, route := range this.rs {
		if method == route.m {
			if route.r.MatchString(uri) {
				route.h(ctx, uri)
				return
			}
		}
	}

	ctx.SetStatusCode(404)
	return
}

/**
 *	Check 回调地址验证
 *
 *	@param ctx				请求上下文
 *	@param url				要验证的回调地址
 *	@param masterSecret		极光渠道masterSecret上
 *
 */
func (this *CallbackApiSvrCommand) Check(ctx *fasthttp.RequestCtx, url string) {
	StatusCode := 200
	respBody := ""
	// 鉴权
	ok := BasicAuth(ctx, *this.channelkey, *this.masterSecret)
	if !ok {
		fmt.Println("BasicAuth fail.")
		StatusCode = 500
	} else {
		args := ctx.QueryArgs()
		echostr := string(args.Peek("echostr")[:])
		fmt.Println("Check", "echostr:", echostr)
		// 原样返回 echostr
		respBody = echostr
		fmt.Println("Check OK.")
	}

	fmt.Println("Check return", StatusCode, respBody)
	ctx.SetStatusCode(StatusCode)
	ctx.WriteString(respBody)
}
